// TARGET DEFINITION
def OUTPUT_DIRECTORY = "output";
def FILE_OUTPUT = OUTPUT_DIRECTORY + "/" + "source_A_import.xml"
def SEARCHED_ID = "DE000AK0A635"; 

// SOURCES DEFINITION
def SOURCES_DIRECTORY = "dataimport/typ_z";
def SOURCES_A_DIRECTORY = "$SOURCES_DIRECTORY/original/";
def SOURCES_B_DIRECTORY = "$SOURCES_DIRECTORY/original/";
def TEMPLATES_DIRECTORY = "$SOURCES_DIRECTORY/templates/";
def BASICS_DIRECTORY = "$SOURCES_DIRECTORY/basics/";
def SOURCES_A_EXTENSION = "xml";
def SOURCES_A_PREFIX = "source_A_";
def SOURCE_B_PREFIX = "source_B.";
def SOURCE_B_SUFFIX = "CSV";
def SOURCE_B_OUTPUT_PREFIX = "source_B";

// DEPENDENCIES/IMPORTS
@Grab(group='org.apache.commons', module='commons-csv', version='1.4')
@Grab(group='org.apache.commons', module='commons-lang3', version='3.5')
@Grab(group='commons-io', module='commons-io', version='2.5')
import java.awt.List
import java.nio.file.CopyOption
import java.nio.file.Files
import java.nio.file.Paths
import java.nio.file.StandardCopyOption
import java.nio.file.CopyMoveHelper.CopyOptions

import org.apache.commons.csv.CSVFormat
import org.apache.commons.csv.CSVParser
import org.apache.commons.csv.CSVRecord
import org.apache.commons.io.FileUtils
import org.apache.commons.io.IOUtils
import org.apache.commons.lang3.StringUtils

import groovy.io.GroovyPrintStream
import groovy.xml.StreamingMarkupBuilder
import groovy.xml.XmlUtil

// NOTE: source A processing
folder = new File(OUTPUT_DIRECTORY)
if(!folder.exists()) {
	folder.mkdir()
} else {
	folder.delete();
	folder.mkdir();
}


def xs = new XmlSlurper();
String nxml = new StreamingMarkupBuilder().bind {
	instruments {
		new File(SOURCES_A_DIRECTORY).eachFileRecurse() { file ->
			if(file.getName().startsWith(SOURCES_A_PREFIX) && file.getName().endsWith(SOURCES_A_EXTENSION)) {
				def p = xs.parse(file);
				def instruments = p.children();
				
				instruments.each { element ->
					if(element.master_data.isin == SEARCHED_ID || element.master_data.wkn == SEARCHED_ID || element.master_data.ssd_id == SEARCHED_ID)
						mkp.yield element
			   }
			}
		}
	}
}

File targetFile = new File(FILE_OUTPUT);
if(!targetFile.exists()) {
	targetFile.createNewFile();
}
XmlUtil.serialize(nxml, new GroovyPrintStream(FILE_OUTPUT, 'utf-8'))


// NOTE: source B processing
new File(SOURCES_B_DIRECTORY).eachFileRecurse() { file ->
	if(file.getName().startsWith(SOURCE_B_PREFIX)) {
		
		def linesToSave = new ArrayList<String>()
		
		def bFileNumber = 1;
		
		new File(SOURCES_B_DIRECTORY, file.getName()).eachLine { line, nb ->
			
			// NOTE: extract header to get file number, which will be used as source_B[BFileNumber].csv
			if(nb == 1) {
				def bHeader = "$line";
				def splitted = bHeader.split("\\.")
				bFileNumber = splitted[1];
				linesToSave << line
			}
			
			// NOTE: keep second line as it is
			if(nb == 2) {
				linesToSave.add("$line")
			}
			
			// NOTE: keep third line as it is
			if(nb == 3) {
				linesToSave.add("$line")
			}
			
			// NOTE: prepare next lines, by searching corresponding ID, columns 3 and 4 are ID's of produkt (WKN/ISIN)
			if(nb > 3) {
				def splitted = line.split(";")
				if(splitted[4].trim() == SEARCHED_ID || splitted[3].trim() == SEARCHED_ID) {
					linesToSave << line
				}
			}
		}
		
		new File(OUTPUT_DIRECTORY, SOURCE_B_OUTPUT_PREFIX + bFileNumber + "." + SOURCE_B_SUFFIX).with { outFile ->
			outFile.withWriter { out ->
				linesToSave.each { out.println it}
			}
		}
	}
}

FileUtils.copyDirectory(new File(TEMPLATES_DIRECTORY), new File(OUTPUT_DIRECTORY))
FileUtils.copyDirectory(new File(BASICS_DIRECTORY), new File(OUTPUT_DIRECTORY))
